# -*- coding: utf-8 -*-
import doctest


# Standard options for DocTests
optionflags = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_ONLY_FIRST_FAILURE)
